package cd.casic.ci.process.process.service.aflManager.impl;

import cd.casic.ci.process.dto.req.aflManager.AflManagerReq;
import cd.casic.ci.process.process.dao.aflManager.AflCrashesInfoDao;
import cd.casic.ci.process.process.dataObject.aflManager.AflCrashesInfo;
import cd.casic.ci.process.process.service.aflManager.AflCrashesInfoService;
import cd.casic.ci.process.util.SftpUploadUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName TargetVersionServiceImpl
 * @Date: 2025/5/17 15:20
 * @Description:
 */
@Service
@Slf4j
public class AflCrashesInfoServiceImpl extends ServiceImpl<AflCrashesInfoDao, AflCrashesInfo> implements AflCrashesInfoService {

    private static final String remoteFilePath = "/home/casic/706/yunqi/";
    private static final String crashesFilePath = "/ai_afl/default/crashes/";


    @Override
    public void saveAflCrashesInfo(AflManagerReq req) {
        try {
            // 步骤1：列出源目录下的所有文件
            List<String> files = SftpUploadUtil.listFilesInRemoteDirectory(
                    req.getMachineHost(), Integer.parseInt(req.getSshPort()), req.getUsername(), req.getPassword(), null, remoteFilePath + "PIP_" + req.getPipelineId() + crashesFilePath);

            if (!CollectionUtils.isEmpty(files)) {
                // 步骤2：批量复制文件到目标目录
                List<String> copiedFiles = SftpUploadUtil.copyRemoteFile(
                        req.getMachineHost(), Integer.parseInt(req.getSshPort()), req.getUsername(), req.getPassword(), null, files, remoteFilePath + "PIP_" + req.getPipelineId() + "/crashes_result/");
                System.out.println("共复制 " + copiedFiles.size() + " 个文件");

                if (!CollectionUtils.isEmpty(copiedFiles)){
                    List<AflCrashesInfo> aflCrashesInfos = new ArrayList<>(0);
                    //新增数据到crashes表中
                    copiedFiles.forEach(o->{
                        AflCrashesInfo aflCrashesInfo = new AflCrashesInfo();
                        aflCrashesInfo.setFilePath(o);
                        aflCrashesInfo.setPipelineId(req.getPipelineId());
                        if (!StringUtils.isEmpty(req.getTaskId())){
                            aflCrashesInfo.setTaskId(req.getTaskId());
                        }
                        if (!StringUtils.isEmpty(req.getPipelineHistoryId())){
                            aflCrashesInfo.setPipelineHistoryId(req.getPipelineHistoryId());
                        }
                        aflCrashesInfos.add(aflCrashesInfo);
                    });
                    baseMapper.insertBatch(aflCrashesInfos);
                }

                // 步骤3：删除旧目录
                SftpUploadUtil.clearRemoteDirectoryFiles(req.getMachineHost(), Integer.parseInt(req.getSshPort()), req.getUsername(), req.getPassword(), null, remoteFilePath + "PIP_" + req.getPipelineId() + crashesFilePath);
                System.out.println("crashes目录下文件已删除: " + remoteFilePath + "PIP_" + req.getPipelineId() + crashesFilePath);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
