package cd.casic.ci.process.process.service.aflManager.impl;

import cd.casic.ci.process.dto.req.aflManager.AflManagerReq;
import cd.casic.ci.process.dto.resp.aflManager.SeedsCountResp;
import cd.casic.ci.process.engine.manager.RunContextManager;
import cd.casic.ci.process.engine.runContext.BaseRunContext;
import cd.casic.ci.process.process.dao.aflManager.AflSeedInfoDao;
import cd.casic.ci.process.process.dataObject.aflManager.AflSeedInfo;
import cd.casic.ci.process.process.service.aflManager.AflSeedInfoService;
import cd.casic.ci.process.util.SftpUploadUtil;
import cd.casic.framework.commons.exception.ServiceException;
import cd.casic.framework.commons.exception.enums.GlobalErrorCodeConstants;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

/**
 * @author HopeLi
 * @version v1.0
 * @ClassName TargetVersionServiceImpl
 * @Date: 2025/5/17 15:20
 * @Description:
 */
@Service
@Slf4j
public class AflSeedInfoServiceImpl extends ServiceImpl<AflSeedInfoDao, AflSeedInfo> implements AflSeedInfoService {

    @Resource
    private RunContextManager runContextManager;

    private static final String remoteFilePath = "/home/casic/706/yunqi/";

    private static final String seedFilePath = "/ai_afl/default/queue/";


    @Override
    public List<SeedsCountResp> findSeedsCount(AflManagerReq req) throws SftpUploadUtil.SftpUploadException {
        List<SeedsCountResp> seedsCounts = new ArrayList<>(0);

        if (StringUtils.isEmpty(req.getPipelineId())){
            throw new ServiceException(GlobalErrorCodeConstants.INTERNAL_SERVER_ERROR.getCode(),"pipelineId不能为空");
        }
        List<String> resultList = SftpUploadUtil.findSeedCount(req.getMachineHost(), Integer.parseInt(req.getSshPort()), req.getUsername(), req.getPassword(), null, remoteFilePath + "PIP_" + req.getPipelineId() + "/ai_afl/default/queue/");

        //解析resultList

        //todo 分区计算交由前端处理了，代码暂且注释掉，若前端无法处理则延续算法后端解决
        //分区计算
        //int timeWindow = 0;
        for (String o : resultList) {
            SeedsCountResp seedsCount = new SeedsCountResp();

            String[] lineParts = o.split("\\s+创建时间:\\s+");
            if (lineParts.length < 2) {
                System.err.println("无效格式: " + o);
                break;
            }

            String fileNamePart = lineParts[0]; // 文件信息部分
            String createTime = lineParts[1];   // 创建时间

            // 提取 rep 值
            int repValue = 0;
            String seedId = "";
            for (String part : fileNamePart.split(",")) {
                if (part.startsWith("rep:")) {
                    repValue = Integer.parseInt(part.substring(4));
                    break;
                }
                if (part.startsWith("id:")) {
                    seedId = part.substring(4);
                    break;
                }
            }

            // 解析创建时间
            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss[.SSSSSSSSS] XXX");
            ZonedDateTime zonedDateTime = ZonedDateTime.parse(createTime, formatter);
            LocalDateTime createDateTime = zonedDateTime.toLocalDateTime();

            // 获取开始时间
            BaseRunContext runContext = runContextManager.getContext(req.getTaskId());
            LocalDateTime startTime = runContext.getStartTime();

            // 计算时间差（分钟）
            Duration duration = Duration.between(startTime, createDateTime);
            int minutesDifference = Math.toIntExact(Math.abs(duration.toMinutes()));

            seedsCount.setRepValue(repValue);
            seedsCount.setMinutesDifference(minutesDifference);
            seedsCount.setSeedId(seedId);
            seedsCounts.add(seedsCount);

//            if (minutesDifference < timeWindow){
//                if (!ObjectUtils.isEmpty(seedsCounts)){
//                    //将数据统计到上一个时间窗口中
//                    SeedsCountResp seedsCountResp = seedsCounts.get(seedsCounts.size() - 1);
//                    Integer value = seedsCountResp.getRepValue();
//                    seedsCountResp.setRepValue(value + repValue);
//                }
//            } else if (minutesDifference > timeWindow){
//                timeWindow = timeWindow + 100;
//                seedsCount.setTimeWindow(timeWindow);
//                seedsCount.setRepValue(repValue);
//                seedsCounts.add(seedsCount);
//
//            }else {
//                if (!ObjectUtils.isEmpty(seedsCounts)){
//                    //将数据统计到上一个时间窗口中
//                    SeedsCountResp seedsCountResp = seedsCounts.get(seedsCounts.size() - 1);
//                    Integer value = seedsCountResp.getRepValue();
//                    seedsCountResp.setRepValue(value + repValue);
//
//                    //同时新增下一个100分钟统计的时间窗口数据
//                    seedsCount.setTimeWindow(timeWindow + 100);
//                    seedsCount.setRepValue(0);
//                    seedsCounts.add(seedsCount);
//                }
//            }
        }
        return seedsCounts;
    }

    @Override
    public void saveAflSeedInfo(AflManagerReq req) {
        try {
            // 步骤1：列出源目录下的所有文件
            List<String> files = SftpUploadUtil.listFilesInRemoteDirectory(
                    req.getMachineHost(), Integer.parseInt(req.getSshPort()), req.getUsername(), req.getPassword(), null, remoteFilePath + "PIP_" + req.getPipelineId() + seedFilePath);

            if (!CollectionUtils.isEmpty(files)) {
                // 步骤2：批量复制文件到目标目录
                List<String> copiedFiles = SftpUploadUtil.copyRemoteFile(
                        req.getMachineHost(), Integer.parseInt(req.getSshPort()), req.getUsername(), req.getPassword(), null, files, remoteFilePath + "PIP_" + req.getPipelineId() + "/seed_result/");
                System.out.println("共复制 " + copiedFiles.size() + " 个文件");

                if (!CollectionUtils.isEmpty(copiedFiles)){
                    List<AflSeedInfo> aflSeedInfos = new ArrayList<>(0);
                    //新增数据到crashes表中
                    copiedFiles.forEach(o->{
                        AflSeedInfo aflSeedInfo = new AflSeedInfo();

                        // 提取文件名部分（queue/之后的内容）
                        String fileNamePart = o.substring(o.indexOf("seed_result/") + 12);

                        // 使用正则匹配 id: 开头的内容
                        java.util.regex.Pattern patternId = java.util.regex.Pattern.compile("id:([^,]+)");
                        java.util.regex.Matcher matcherId = patternId.matcher(fileNamePart);

                        // 使用正则匹配 rep: 开头的内容
                        java.util.regex.Pattern patternRep = java.util.regex.Pattern.compile("rep:(\\d+)");
                        java.util.regex.Matcher matcherRep = patternRep.matcher(fileNamePart);

                        if (matcherRep.find()) {
                            String repValue = matcherRep.group(1);
                            aflSeedInfo.setSeedAmount(repValue);
                        }

                        if (matcherId.find()) {
                            String idValue = matcherId.group(1);
                            aflSeedInfo.setSeedId(idValue);
                        }

                        aflSeedInfo.setFilePath(o);
                        if (!StringUtils.isEmpty(req.getTaskId())){
                            aflSeedInfo.setTaskId(req.getTaskId());
                        }
                        if (!StringUtils.isEmpty(req.getPipelineHistoryId())){
                            aflSeedInfo.setPipelineHistoryId(req.getPipelineHistoryId());
                        }
                        aflSeedInfo.setPipelineId(req.getPipelineId());

                        aflSeedInfos.add(aflSeedInfo);
                    });
                    baseMapper.insertBatch(aflSeedInfos);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
